#define NETBSD_INSPIRED 1
#define STD_INSPIRED 1
#include "locale_intern.h"
#include "private.h"

void dprintf(const char *, ...);

extern struct SignalSemaphore timezonelock;
extern timezone_t systemtimezone;

APTR TZAlloc(CONST_STRPTR timezone, struct LocaleBase *LocaleBase)
{
  return tzalloc(timezone);
}

void TZFree(APTR sp, struct LocaleBase *LocaleBase)
{
  tzfree(sp);
}

struct tm *TZLocalTime(APTR sp, ULONG t, struct tm *tmp, struct LocaleBase *LocaleBase)
{
  struct tm *rc;
  if (!tmp)
    return NULL;
  if (sp)
    rc = localtime_rz(sp, &t, tmp);
  else
  {
    Forbid();
    rc = localtime_rz(systemtimezone, &t, tmp);
    Permit();
  }
  return rc;
}

ULONG TZMKTime(APTR sp, struct tm *tm, struct LocaleBase *LocaleBase)
{
  ULONG rc;
  if (!tm)
    return ~0;
  if (sp)
    rc = (ULONG) mktime_z(sp, tm);
  else
  {
    /*
       This codepath attempts to be safe from semaphore lockups / recursion
       issues:
       1) It uses Forbid() to make it less b0rk when called from timer.device etc.
       2) Everything should be preloaded to memory at this stage so that no
          Wait() will be called or recursion will occurt from loading resources
          from a file system.

       Yes, it is a bit delicate and depends on the tzdb internals. - Piru
    */
    Forbid();
    if (0) {
      char buf[40];
      asctime_r(tm, buf); buf[strlen(buf) - 1] = '\0';
      dprintf("%s: convert local time %d.%d.%d %02d:%02d:%02d (tm_isdst %d) (%s) using tz %p to unix timestamp\n",
              __func__, tm->tm_mday, tm->tm_mon + 1, tm->tm_year + 1900,
              tm->tm_hour, tm->tm_min, tm->tm_sec, tm->tm_isdst, buf, systemtimezone);
    }
    rc = (ULONG) mktime_z(systemtimezone, tm);
    //dprintf("%s: resulting unix timestamp %lu\n", __func__, rc);
    Permit();
  }
  return rc;
}

ULONG TZTime2Posix(APTR sp, ULONG t, struct LocaleBase *LocaleBase)
{
  ULONG rc;
  if (sp)
    rc = (ULONG) time2posix_z(sp, (time_t) t);
  else
  {
    Forbid();
    rc = time2posix_z(systemtimezone, (time_t) t);
    Permit();
  }
  return rc;
}

ULONG TZPosix2Time(APTR sp, ULONG t, struct LocaleBase *LocaleBase)
{
  ULONG rc;
  if (sp)
    rc = (ULONG) posix2time_z(sp, (time_t) t);
  else
  {
    Forbid();
    rc = posix2time_z(systemtimezone, (time_t) t);
    Permit();
  }
  return rc;
}

STRPTR TZAscTime(const struct tm *tm, STRPTR buf, struct LocaleBase *LocaleBase)
{
  return (STRPTR) asctime_r(tm, (char *) buf);
}

STRPTR TZCTime(ULONG t, STRPTR buf, struct LocaleBase *LocaleBase)
{
  return (STRPTR) ctime_r((time_t *) &t, (char *) buf);
}

struct tm *TZGMTime(ULONG t, struct tm *tm, struct LocaleBase *LocaleBase)
{
  struct tm *rc;
  ObtainSemaphore(&timezonelock);
  rc = gmtime_r((time_t *) &t, tm);
  ReleaseSemaphore(&timezonelock);
  return rc;
}
